@@ -4,13 +4,6 @@ class NoSufficientContractorError(Exception):
 
     It indicates when the contractors have not sufficient resources to perform schedule.
     """
-
-    def __init__(self, message):
-        super().__init__(message)
-
-
-class NoAvailableResources(Exception):
-
     def __init__(self, message):
         super().__init__(message)
 
@@ -21,13 +14,7 @@ class NotEnoughMaterialsInDepots(Exception):
         super().__init__(message)
 
 
-class NoDepots(Exception):
-
-    def __init__(self, message):
-        super().__init__(message)
-
-
-class IncorrectAmountOfWorker(Exception):
+class NoAvailableResources(Exception):
 
     def __init__(self, message):
         super().__init__(message)
