@@ -5,9 +5,7 @@ from functools import cached_property, cache
 from random import Random
 from typing import Optional
 
-import dill
 import numpy as np
-import pandas as pd
 from scipy.sparse import dok_matrix
 
 from sampo.schemas import uuid_str
@@ -62,25 +60,21 @@ class GraphNode(JSONSerializable['GraphNode']):
     def __hash__(self) -> int:
         return hash(self.id)
 
-    def __eq__(self, other) -> bool:
-        if other is None:
-            return False
-        return self.id == other.id
-
     def __repr__(self) -> str:
         return self.id
 
-    # def __getstate__(self):
-    #     # custom method to avoid calling __hash__() on GraphNode objects
-    #     return self._serialize()
-    #
-    # def __setstate__(self, state):
-    #     # custom method to avoid calling __hash__() on GraphNode objects
-    #     representation = self._deserialize(state)
-    #     self.__init__(representation['work_unit'],
-    #                   representation['parent_edges'])
-    #     # self._work_unit = representation['work_unit']
-    #     # self._parent_edges = [GraphEdge(*e) for e in representation['parent_edges']]
+    def __getstate__(self):
+        # custom method to avoid calling __hash__() on GraphNode objects
+        return self._work_unit._serialize(), \
+            [(e.start.id, e.lag, e.type.value) for e in self._parent_edges]
+
+    def __setstate__(self, state):
+        # custom method to avoid calling __hash__() on GraphNode objects
+        s_work_unit, s_parent_edges = state
+        self.__init__(WorkUnit._deserialize(s_work_unit),
+                      s_parent_edges)
+        # self._work_unit = representation['work_unit']
+        # self._parent_edges = [GraphEdge(*e) for e in representation['parent_edges']]
 
     def _serialize(self) -> T:
         return {
@@ -108,13 +102,11 @@ class GraphNode(JSONSerializable['GraphNode']):
         edges: list[GraphEdge] = []
         if parent_works:
             if isinstance(parent_works[0], GraphNode):
-                edges = [GraphEdge(p, self, 0, EdgeType.FinishStart) for p in parent_works]
+                edges = [GraphEdge(p, self, 1, EdgeType.FinishStart) for p in parent_works]
             elif isinstance(parent_works[0], tuple):
                 edges = [GraphEdge(p, self, lag, edge_type) for p, lag, edge_type in parent_works]
 
         for edge, parent in zip(edges, parent_works):
-            if isinstance(parent[0] if isinstance(parent, tuple) else parent, str):
-                print(f'{parent} {edge}')
             parent: GraphNode = parent[0] if isinstance(parent, tuple) else parent
             parent._add_child_edge(edge)
             parent.invalidate_children_cache()
@@ -126,14 +118,14 @@ class GraphNode(JSONSerializable['GraphNode']):
         self.__dict__.pop('parents_set', None)
         self.__dict__.pop('inseparable_parent', None)
         self.__dict__.pop('inseparable_son', None)
-        self.get_inseparable_chain.cache_clear()
+        self.__dict__.pop('get_inseparable_chain', None)
 
     def invalidate_children_cache(self):
         self.__dict__.pop('children', None)
         self.__dict__.pop('children_set', None)
         self.__dict__.pop('inseparable_parent', None)
         self.__dict__.pop('inseparable_son', None)
-        self.get_inseparable_chain.cache_clear()
+        self.__dict__.pop('get_inseparable_chain', None)
 
     def is_inseparable_parent(self) -> bool:
         return self.inseparable_son is not None
@@ -288,12 +280,12 @@ class GraphNode(JSONSerializable['GraphNode']):
                     for edge in self.edges_to if edge.start in node2swork), default=Time(0))
 
 
-def get_start_stage(work_id: str | None = None, rand: Random | None = None) -> GraphNode:
+def get_start_stage(work_id: str | None = "", rand: Random | None = None) -> GraphNode:
     """
     Creates a service vertex necessary for constructing the graph of works,
     which is the only vertex without a parent in the graph
     :param work_id: id for the start node
-    :param rand: number generator with a fixed seed, or None for no fixed seed
+    :param rand: Number generator with a fixed seed, or None for no fixed seed
     :return: desired node
     """
     work_id = work_id or uuid_str(rand)
@@ -301,20 +293,18 @@ def get_start_stage(work_id: str | None = None, rand: Random | None = None) -> G
     return GraphNode(work, [])
 
 
-def get_finish_stage(parents: list[GraphNode | tuple[GraphNode, float, EdgeType]], work_id: str | None = None,
+def get_finish_stage(parents: list[GraphNode | tuple[GraphNode, float, EdgeType]], work_id: str | None = "",
                      rand: Random | None = None) -> GraphNode:
     """
     Creates a service vertex necessary for constructing the graph of works,
     which is the only vertex without children in the graph
     :param parents: a list of all non-service nodes that do not have children
     :param work_id: id for the start node
-    :param rand: number generator with a fixed seed, or None for no fixed seed
+    :param rand: Number generator with a fixed seed, or None for no fixed seed
     :return: desired node
     """
     work_id = work_id or uuid_str(rand)
-    finish_priority = max(node.work_unit.priority for node in parents)
-    work = WorkUnit(str(work_id), 'finish of project', [], group='service_works',
-                    priority=finish_priority, is_service_unit=True)
+    work = WorkUnit(str(work_id), 'finish of project', [], group='service_works', is_service_unit=True)
     return GraphNode(work, parents)
 
 
@@ -355,75 +345,13 @@ class WorkGraph(JSONSerializable['WorkGraph']):
         finish = get_finish_stage(parents=without_successors, rand=rand)
         return WorkGraph(start, finish)
 
-    def to_frame(self, save_req=False) -> pd.DataFrame:
-        # Define the format of the output DataFrame
-        graph_df_structure = {'activity_id': [],
-                              'activity_name': [],
-                              'granular_name': [],
-                              'volume': [],
-                              'measurement': [],
-                              'priority': [],
-                              'predecessor_ids': [],
-                              'connection_types': [],
-                              'lags': []}
-
-        if save_req:
-            graph_df_structure['min_req'] = []
-            graph_df_structure['max_req'] = []
-            graph_df_structure['req_volume'] = []
-
-        # List of service 'start' and 'finish' nodes, which will not be included to the project's DataFrame
-        start_finish_service_nodes = [self.start.id, self.finish.id]
-
-        for node in self.nodes:
-            if node.id not in start_finish_service_nodes:
-                node_info_dict = node.dumpd()
-
-                # Get information about tasks from graph nodes (work units)
-                graph_df_structure['activity_id'].append(node_info_dict['work_unit']['id'])
-                graph_df_structure['activity_name'].append(node_info_dict['work_unit']['display_name'])
-                graph_df_structure['granular_name'].append(node_info_dict['work_unit']['name'])
-                graph_df_structure['volume'].append(node_info_dict['work_unit']['volume'])
-                graph_df_structure['measurement'].append(node_info_dict['work_unit']['volume_type'])
-                graph_df_structure['priority'].append(node_info_dict['work_unit']['priority'])
-
-                if save_req:
-                    min_req_dict = dict()
-                    max_req_dict = dict()
-                    req_volume_dict = dict()
-
-                    for req in node_info_dict['work_unit']['worker_reqs']:
-                        min_req_dict[req['kind']] = req['min_count']
-                        max_req_dict[req['kind']] = req['max_count']
-                        req_volume_dict[req['kind']] = req['volume']['value']
-
-                    graph_df_structure['min_req'].append(min_req_dict)
-                    graph_df_structure['max_req'].append(max_req_dict)
-                    graph_df_structure['req_volume'].append(req_volume_dict)
-
-                # Get information about connections between tasks from parent edges
-                predecessors_lst = []
-                connection_types_lst = []
-                lags_lst = []
-                for predecessor_info in node_info_dict['parent_edges']:
-                    if predecessor_info[0] not in start_finish_service_nodes:
-                        predecessors_lst.append(str(predecessor_info[0]))
-                        lags_lst.append(str(predecessor_info[1]))
-                        connection_types_lst.append(str(predecessor_info[2]))
-
-                graph_df_structure['predecessor_ids'].append(','.join(predecessors_lst))
-                graph_df_structure['lags'].append(','.join(lags_lst))
-                graph_df_structure['connection_types'].append(','.join(connection_types_lst))
-
-        return pd.DataFrame.from_dict(graph_df_structure)
-
-    def __hash__(self) -> int:
+    def __hash__(self):
         return hash(self.start) + 17 * hash(self.finish)
 
     def __getitem__(self, item: str) -> GraphNode:
         return self.dict_nodes[item]
 
-    def __getstate__(self) -> dict:
+    def __getstate__(self):
         # custom method to avoid calling __hash__() on GraphNode objects
         representation = self._serialize()
         representation['start'] = self.start.id
@@ -483,13 +411,3 @@ class WorkGraph(JSONSerializable['WorkGraph']):
                 adj_mx[i, c_i] = weight
 
         return ordered_nodes, adj_mx, id2node
-
-
-def recreate(state):
-    # custom method to avoid calling __hash__() on GraphNode objects
-    return WorkGraph._deserialize(state)
-
-# TODO Check all types for dill-serializability and make dedicated file with serializers
-@dill.register(WorkGraph)
-def serialize_wg(pickler, obj):
-    pickler.save_reduce(recreate, (obj._serialize(),), obj=obj)
